/*****************************************************************

docbegin:


*  Name of source file:         j_ima_mosaic_main.c

*  Version of source file:      7.0.1

*  Parent component:            None

*  Programmer:                  Jrme Chenevez

*  Affiliation:                 Danish National Space Center

*  Contact address:             Juliane Maries Vej 30
*                               DK-2100 Copenhagen, Denmark
*                               Phone: +45 35325703  FAX: +45 35362475
*                               jerome@dnsc.dk

*  Purpose:                     Read a set of sky images for an observation group
*                               and produce the weighted mosaic image

*  Origin date:                 20041206

*  Update history:              5.0    20041206  Copied from mosaic_weight-5.2 (see pre-history)
*				5.0.1  20050419  Now also combine iros input images in cnts/cm2/s
*				5.0.2  20050603  Urgent fixes for OSA 5 by SP 
*                                                SPRs: 4174 (part.), 4176 (part.), 4190, 4191, 4205
*                               5.0.3  20050620  Fix SPRs 4174, 4175 4176, 4222
*                               5.0.4  20050622  Fix SPR 4244
*                               5.1.0  20051014  SCREW 1772 : base the image selection on VARIANCE maps
*                               6.0.0  20060329  Fix SPR 4377 : resolve energy selection problem
*						 SCREW 1850 : Use correct WCS routines allowing large mosaics
*                               6.0.1  20060908  Fix SPR 4301 : new parameter dolBPL
*                               6.0.2  20060915  SCRW 1918 : Copy j_ima_iros IMATYPE keywords to output
*                               6.0.3  20060927  SCREW 1929 : Gather all energy bands and mosaic products in one single output file
*                               6.0.4  20061110  SPR 4603 : Copy correct input image units (cnts/dm2/s) to output
*                               6.0.5  20061206  SPR 4625 : Some selection strings defined too short
*                                                SPR 4632 : Allow variable number of input images by dynamic memory allocation
*                               7.0.0  20070707  SCREW 2008 Allow mixing of JMX1 and JMX2 mosaics
*						 SCREW 2024 Optionnal printing of input ScW list in the HEADERs of mosaic file
*                               7.0.1  20070820  SPR 4711: Allow empty dolBPL when combining Jem-X mosaics

*
* Pre-history:
* Routine to compute mosaic weigthed sky images. Jrme Chenevez, DSRI.
* Version 0   Sept 2003 : Experimental version (memory problems solved by declaration of big arrays outside the main pgm).
* Version 1   Oct  2003 : First release (the user has to set input images dimensions "by hand" - solved in next versions).
* Version 2   Dec  2003 : Use of a parameter file. New weighting method by NJW. Introduction of larger input images from CBJ.
* Version 3   Feb  2004 : Improved selection capabilities for NL input images. Ranking of weights for a given sky position.
* Version 4   Apr  2004 : Advanced I/O formats following agreements described in JEM-X TN-14.
* Version 5   Aug  2004 : Implementation of NJW's subroutine to compute variance maps. The keyword VIGNDOL may be defined by an environment variable.
* Version 5.1 Nov  2004 : Use of CFITSIO WCS - OK for OSA4.x images. Still problems with IROS images, so only applied if the mosaic > 20 degrees.
* Version 5.2 Dec  2004 : New parameter vignMix to allow combination of both JEM-X1&2 images.
   
*  Type of element:             Function

*  Part of package:             j_ima_mosaic

*  Name:                        j_ima_mosaic

*  Calls:

*  Returns:

*  Required components:         DAL, PIL, RIL, DAL3GEN, CFITSIO

docend:

Various remarks:

Flow of program:

Error codes:

   Basic offset:
   J_ERROR_CODE_START

****************************************************************/

#include "j_ima_mosaic.h"

int main (int argc, char *argv[])
{
  char groupDOL[DAL_FILE_NAME_STRING];
  char imSelect[DAL_FILE_NAME_STRING];
  char dolBPL[DAL_FILE_NAME_STRING];
  char mosaic[DAL_FILE_NAME_STRING];
  double radius;
  double emin;
  double emax;
  double diamout;
  double cdelt;
  double centerA;
  double centerD;
  int plot_T;
  int plot_I;
  int plot_V;
  int plot_S;
  int liScW = 0 ;  
  int comos;
  int status = ISDC_OK;
  int mode = 0;            /*  Running mode of this executable */
  int chatter = 0;         /*  Level of debugging/monitoring output */
/*  int clobber = 0;           (Boolean) */
 
  do {

    /* Use a once-only do/while loop to force program termination whenever
       status returned is not ISDC_OK  */
    
/*
 *
 *         Initialisation
 *
 */

    mode = CommonInit(COMPONENT_NAME, COMPONENT_VERSION,argc,argv);
    if (mode != ISDC_SINGLE_MODE) {
      RILlogMessage(NULL, Error_2,
                "Not started in ISDC_SINGLE_MODE, mode = %d", mode);
      status = J_ERROR_CODE_START;
      continue;
    }

/*
 *
 *       Get the parameters
 *
 */

    status = j_ima_mosaic_params(groupDOL,
                                 imSelect,
                                 dolBPL,
                                 mosaic,
                                 &radius,
                                 &emin,
                                 &emax,
                                 &diamout,
                                 &cdelt, 
                                 &centerA,
                                 &centerD,
                                 &plot_T, 
                                 &plot_I,
                                 &plot_V,
                                 &plot_S,
                                 &liScW,
                                 &comos,
                             /*  &clobber,  */
                                 &chatter,
                                 status);

    if (status != ISDC_OK) {
    	RILlogMessage(NULL, Error_2, 
    		"Problem getting parameters etc., status = %d\n", status);
      continue;
    }



/*
 *
 *     Go ahead and do the work 
 *
 *
 */

    status = j_ima_mosaic_work(groupDOL,
                               imSelect,
                               dolBPL,
                               mosaic,
                               radius,
                               emin,
                               emax,
                               diamout,
                               cdelt, 
                               centerA,
                               centerD,
                               plot_T, 
                               plot_I,
                               plot_V,
                               plot_S,
                               liScW,
                               comos,
                            /* clobber,  */
                               chatter,
                               status);

    if (status != ISDC_OK) {
    	RILlogMessage(NULL, Error_2, 
    		"Problem in the work function, status = %d\n", status);
      continue;
    }

    
    RILlogMessage(NULL, Log_2,"Work completed: status is %d\n", status);

   } while (0);
	

   if( status != ISDC_OK ) {
       RILlogMessage(NULL, Error_2,
          "Terminating with status = %d", status);
   }

  
   CommonExit(status);

}
